import { useState } from 'react'
import Taro, { useRouter } from '@tarojs/taro'
import Navbar from '@/components/Navbar'
import { View, Image, Button, Input } from '@tarojs/components'
import { getTopSafeHeight, mediaRotate, squaredImageUrl, filterCoverPath } from '@/utils'
import { uploadImageFile } from '@/utils/oss'
import { userInfoUpdateUrl } from '@/http/api'
import './editUpload.less'

type UploadFileResultType = {
    fileUrl: string
    objectKey: string
    host: string
    relativeUrl: string
}

function editUpload() {
    const { params } = useRouter()
    const avatarUrl = decodeURIComponent(params.avatarUrl || '')
    const title = params.type === 'avatar' ? '设置头像' : '设置昵称'
    const navigationColor = params.type === 'avatar' ? '#fff' : ''
    const textColor = params.type === 'avatar' ? '#fff' : '#000'

    return (
        <View className='user-setting-page' style={{
            background: params.type === 'avatar' ? '#000' : ''
        }}>
            <Navbar title={title} navigationColor={navigationColor} textColor={textColor} status navigation />
            {params.type === 'avatar' && <Avatar avatarUrl={avatarUrl} nickName={params.nickName} />}
            {params.type === 'nickname' && <UserName avatarUrl={avatarUrl} nickName={params.nickName} />}
        </View>
    )
}

function Avatar({ avatarUrl, nickName }) {
    const [avatar, setAvatarUrl] = useState(avatarUrl)
    const { statusBarHeight, navigationBarHeight } = getTopSafeHeight()
    const style = {
        height: `calc(100vh - ${statusBarHeight + navigationBarHeight}px)`
    }

    function onChooseAvatar(res) {
        const { detail } = res
        const { avatarUrl } = detail || {}

        Taro.getImageInfo({
            src: avatarUrl,
            success(res) {
                updateUserSetting(res)
            }
        })
    }

    function updateUserSetting(imageInfo) {
        const { path } = imageInfo

        uploadImageFile({
            tempFilePath: path
        }).then((res: UploadFileResultType) => {
            const { relativeUrl, host } = res
            const rotate = mediaRotate(imageInfo.orientation)
            const width = Math.min(Math.min(imageInfo.width, imageInfo.height), 4000)
            const squared = squaredImageUrl(relativeUrl, rotate, width)
            const fullSquaredUrl = filterCoverPath(host + squared)
            setAvatarUrl(fullSquaredUrl)

            Taro.showToast({
                title: '上传成功',
                icon: 'success'
            })
        }).catch(() => {
            Taro.showToast({
                title: '上传失败',
                icon: 'error'
            })
        })
    }

    function saveAvatar() {
        Taro.$http.post(userInfoUpdateUrl, {
            avatarUrl: avatar,
            nickName
        }).then((res: RequestType) => {
            const { code } = res
            if (code !== 0) {
                Taro.showToast({
                    title: '修改失败',
                    icon: 'error'
                })
                return
            }

            Taro.showToast({
                title: '修改成功'
            })
        })
    }

    return (
        <View className='avatar' style={style}>
            <View className='outer-box'>
                <Image src={avatar} mode='widthFix' />
                <View className='mask'></View>
            </View>

            <View className='operation'>
                <Button className='update-avatar' openType='chooseAvatar' onChooseAvatar={onChooseAvatar}>
                    更换头像
                </Button>

                <Button className='save-avatar' onClick={saveAvatar}>
                    保存头像
                </Button>
            </View>
        </View>
    )
}

function UserName({ avatarUrl, nickName }) {
    const [name, setName] = useState(nickName)
    function onInput(res) {
        const { detail } = res
        const { value } = detail
        setName(value)
    }

    function saveName() {
        Taro.$http.post(userInfoUpdateUrl, {
            avatarUrl,
            nickName: name
        }).then((res: RequestType) => {
            const { code } = res
            if (code !== 0) {
                Taro.showToast({
                    title: '修改失败',
                    icon: 'error'
                })
                return
            }

            Taro.showToast({
                title: '修改成功'
            })
        })
    }

    return (
        <View className='username'>
            <Input value={name} onInput={onInput} maxlength={50} />
            <View className='tips'>修改昵称，最多50个字</View>
            <Button className='save-btn' onClick={saveName}>保存</Button>
        </View>
    )
}

export default editUpload
